import enum
import os

from utils.errors.database_config_error import DatabaseConfigError
from dotenv import load_dotenv
import logging


class DatabaseConfig():
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Parsing .env")

        anything_set = load_dotenv()

        if anything_set:
            self.logger.debug(".env found")
        else:
            self.logger.error(".env not found")
            raise FileNotFoundError(".env not found.")

        host = os.getenv("DATABASE_HOST")
        port = os.getenv("DATABASE_PORT", "3306")
        name = os.getenv("DATABASE_NAME")
        user = os.getenv("DATABASE_USER")
        password = os.getenv("DATABASE_PASSWORD")

        self.logger.debug("Validating fetched values from .env")

        if host is None or host == "":
            self.logger.error("DATABASE_HOST is empty")
            raise DatabaseConfigError(
                "Config is invalid.", "DATABASE_HOST", "(Empty)")
        if name is None or name == "":
            self.logger.error("DATABASE_NAME is empty")
            raise DatabaseConfigError(
                "Config is invalid.", "DATABASE_NAME", "(Empty)")
        if user is None or user == "":
            self.logger.error("DATABASE_USER is empty")
            raise DatabaseConfigError(
                "Config is invalid.", "DATABASE_USER", "(Empty)")
        if password is None or password == "":
            self.logger.error("DATABASE_PASSWORD is empty")
            raise DatabaseConfigError(
                "Config is invalid.", "DATABASE_PASSWORD", "(Empty)")

        if not port.isdigit():
            self.logger.error("DATABASE_PORT is invalid. Not a number")
            raise DatabaseConfigError(
                "Config is invalid", "DATABASE_PORT", port)

        self.logger.debug("All config validated")

        self.host = host
        self.port = port
        self.name = name
        self.user = user
        self.password = password


class TransactionLevel(enum.Enum):
    insecure = 'READ UNCOMMITED'
    secure = 'SERIALIZABLE'


class UserConfig():
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if not hasattr(self, "transaction_level"):
            self.transaction_level = TransactionLevel.insecure
