import logging

from sqlalchemy.orm import sessionmaker, Session
from sqlalchemy import create_engine

from utils.config import DatabaseConfig
from utils.errors.database_config_error import DatabaseConfigError


class DatabaseManager():

    _instance: 'DatabaseManager' = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self) -> None:
        self.logger = logging.getLogger(__name__)
        self.logger.info("Reading database config")
        self.database_config = DatabaseConfig()
        self.engine = create_engine(f'mysql+mysqlconnector://{self.database_config.user}:{
                                    self.database_config.password}@{self.database_config.host}/{self.database_config.name}')
        self.session_local = sessionmaker(bind=self.engine)

    def cleanup(self) -> None:
        self.logger.debug("Closing connection")
        self.engine.dispose()

    @classmethod
    def get_session(cls) -> Session:
        return DatabaseManager._instance.session_local()
